"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var history_1 = require("history");
var path_to_regexp_1 = require("path-to-regexp");
var ForgetRegisterError_1 = require("@redux-model/web/core/exceptions/ForgetRegisterError");
var web_1 = require("@redux-model/web");
var history_2 = require("./history");
var LISTEN_ALL = {};
var RouterModel = /** @class */ (function (_super) {
    tslib_1.__extends(RouterModel, _super);
    function RouterModel() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.isInitialized = false;
        _this.pathListeners = [];
        _this.changeHistory = _this.action(function (_, payload) {
            return payload;
        });
        _this.push = function (path, state) {
            _this.getHistory().push(path, state);
        };
        _this.replace = function (path, state) {
            _this.getHistory().replace(path, state);
        };
        _this.go = function (index) {
            _this.getHistory().go(index);
        };
        _this.goBack = function () {
            _this.getHistory().goBack();
        };
        _this.goForward = function () {
            _this.getHistory().goForward();
        };
        return _this;
    }
    RouterModel.prototype.listenPath = function (path, fn) {
        var token = "un_" + this.pathListeners.length + "_" + Math.random();
        var keys = [];
        var reg = path_to_regexp_1.pathToRegexp(path, keys);
        var subscriber = { path: path, fn: fn, reg: reg, keys: keys, token: token };
        this.pathListeners.push(subscriber);
        if (this.isInitialized) {
            this.publishOne(subscriber, this.data.location, this.data.action);
        }
        return token;
    };
    RouterModel.prototype.listenAll = function (fn) {
        var token = "un_" + this.pathListeners.length + "_" + Math.random();
        var subscriber = { path: LISTEN_ALL, fn: fn, token: token };
        this.pathListeners.push(subscriber);
        if (this.isInitialized) {
            this.publishOne(subscriber, this.data.location, this.data.action);
        }
        return token;
    };
    RouterModel.prototype.unlisten = function (token) {
        this.pathListeners = this.pathListeners.filter(function (item) {
            return item.token !== token;
        });
    };
    RouterModel.prototype.registerBrowser = function (history) {
        var originalHistory = history_2.getHistory();
        var newHistory = history || originalHistory || history_1.createBrowserHistory();
        history_2.setHistory(newHistory);
        if (originalHistory && originalHistory !== newHistory && this.unregister) {
            this.unregister();
        }
        return this.register();
    };
    RouterModel.prototype.registerHash = function (history) {
        var originalHistory = history_2.getHistory();
        var newHistory = history || originalHistory || history_1.createHashHistory();
        history_2.setHistory(newHistory);
        if (originalHistory && originalHistory !== newHistory && this.unregister) {
            this.unregister();
        }
        return this.register();
    };
    RouterModel.prototype.getHistory = function () {
        var history = history_2.getHistory();
        if (!history) {
            throw new ForgetRegisterError_1.ForgetRegisterError('RouterModel');
        }
        return history;
    };
    RouterModel.prototype.register = function () {
        var _this = this;
        var history = history_2.getHistory();
        if (!history) {
            throw new ReferenceError('Use "registerBrowser()" or "registerHash()" for routerModel.');
        }
        if (!this.unregister) {
            this.unregister = history.listen(function (location, action) {
                _this.changeHistory({
                    location: location,
                    action: action,
                });
                _this.publishAll(location, action);
            });
        }
        return _super.prototype.register.call(this);
    };
    RouterModel.prototype.publishAll = function (location, action) {
        var _this = this;
        this.pathListeners.forEach(function (subscriber) {
            _this.publishOne(subscriber, location, action);
        });
    };
    RouterModel.prototype.publishOne = function (subscriber, location, action) {
        if (subscriber.path === LISTEN_ALL) {
            return subscriber.fn(location, action);
        }
        var result = subscriber.reg.exec(location.pathname);
        if (result === null) {
            return;
        }
        var params = {};
        subscriber.keys.forEach(function (_a, index) {
            var name = _a.name;
            params[name] = result[index + 1];
        });
        subscriber.fn(params, location, action);
    };
    RouterModel.prototype.onReducerCreated = function (store) {
        _super.prototype.onReducerCreated.call(this, store);
        this.publishAll(this.data.location, this.data.action);
        this.isInitialized = true;
    };
    RouterModel.prototype.initReducer = function () {
        var history = this.getHistory();
        return {
            location: history.location,
            action: history.action,
        };
    };
    RouterModel.prototype.autoRegister = function () {
        return false;
    };
    return RouterModel;
}(web_1.Model));
exports.routerModel = new RouterModel();
